<?php
header("Content-Type: application/json; charset=UTF-8");
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: PUT");

// فایل‌های مورد نیاز
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/config/core.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/config/Database.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/auth/class.Utility.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/libs/php-jwt-master/src/JWT.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/libs/php-jwt-master/src/ExpiredException.php';

use \Firebase\JWT\JWT;
use \Firebase\JWT\ExpiredException;

$utility = new Utility();

// دریافت توکن از هدر Authorization
$encodedString = $utility->getToken();
if (!$encodedString) {
    $utility->handleError(401, "توکن ارائه نشده است.");
    exit();
}

// دریافت داده‌های ورودی از بدنه درخواست
$data = json_decode(file_get_contents("php://input"));

// اعتبارسنجی داده‌های ورودی
if (
    empty($data->tracking_code) ||
    empty($data->action) || // action می‌تواند 'accept' یا 'reject' باشد
    ($data->action === 'accept' && (empty($data->payment_date) || empty($data->payment_time))) // برای قبول کردن، payment_date و payment_time الزامی است
) {
    $utility->handleError(400, "لطفاً تمام فیلدها را پر کنید.");
    exit();
}

try {
    // بررسی اعتبار توکن
    $decodedToken = JWT::decode($encodedString, JWT_KEY, ['HS256']);
    $decodedPayload = (array) $decodedToken->data;

    // سطوح دسترسی مجاز
    $allowedAccessLevels = ['manager', '1'];

    // بررسی دسترسی
    if (!isset($decodedPayload['access_level']) || !in_array($decodedPayload['access_level'], $allowedAccessLevels)) {
        $utility->handleError(403, "شما دسترسی لازم برای این عملیات را ندارید.");
        exit();
    }

    // اتصال به دیتابیس
    $database = new Database();
    $conn = $database->getConnection();

    // شناسه کاربر از توکن
    $userId = $decodedPayload['id'] ?? null;

    if (!$userId) {
        $utility->handleError(400, "شناسه کاربری در توکن وجود ندارد.");
        exit();
    }

    // بررسی وجود درخواست بر اساس tracking_code و فراخوانی account_type
    $checkRequestQuery = "SELECT id, point, phone, account_type 
                          FROM transaction 
                          WHERE tracking_code = :tracking_code 
                            AND payment_status = 'در انتظار' 
                          LIMIT 1";
    $checkStmt = $conn->prepare($checkRequestQuery);
    $checkStmt->bindValue(":tracking_code", $data->tracking_code, PDO::PARAM_STR);
    $checkStmt->execute();
    $requestData = $checkStmt->fetch(PDO::FETCH_ASSOC);

    if (!$requestData) {
        $utility->handleError(404, "درخواستی با این کد پیگیری یافت نشد یا قبلاً پرداخت شده/رد شده است.");
        exit();
    }

    // اطلاعات درخواست
    $requestPoint = intval($requestData['point']); // امتیاز درخواستی
    $userPhone = $requestData['phone'];             // شماره موبایل کاربر
    $accountType = $requestData['account_type'];      // نوع حساب: 'admin' یا 'user'

    // شروع تراکنش
    $conn->beginTransaction();

    if ($data->action === 'accept') {
        // اعتبارسنجی تاریخ و زمان پرداخت
        if (empty($data->payment_date) || empty($data->payment_time)) {
            $utility->handleError(400, "برای قبول کردن درخواست، تاریخ و زمان پرداخت الزامی است.");
            exit();
        }

        // تبدیل تاریخ و زمان پرداخت به فرمت MySQL (YYYY-MM-DD HH:MM:SS)
        $paymentDateTime = DateTime::createFromFormat('Y-m-d H:i:s', $data->payment_date . ' ' . $data->payment_time);
        if (!$paymentDateTime) {
            $utility->handleError(400, "فرمت تاریخ و زمان پرداخت نامعتبر است. فرمت صحیح: YYYY-MM-DD HH:MM:SS");
            exit();
        }
        $paymentDateTimeFormatted = $paymentDateTime->format('Y-m-d H:i:s');

        // به‌روزرسانی وضعیت پرداخت به "پرداخت شده"
        $updateQuery = "UPDATE transaction 
                        SET payment_status = 'پرداخت شده', 
                            date_payment = :payment_date, 
                            time_payment = :payment_time, 
                            description = :description 
                        WHERE tracking_code = :tracking_code 
                          AND payment_status = 'در انتظار'";
        $updateStmt = $conn->prepare($updateQuery);
        $updateStmt->bindValue(":tracking_code", $data->tracking_code, PDO::PARAM_STR);
        $updateStmt->bindValue(":payment_date", $data->payment_date, PDO::PARAM_STR);
        $updateStmt->bindValue(":payment_time", $data->payment_time, PDO::PARAM_STR);
        $updateStmt->bindValue(":description", $data->description ?? '', PDO::PARAM_STR);

        if ($updateStmt->execute() && $updateStmt->rowCount() > 0) {
            // انتخاب جدول مناسب بر اساس نوع حساب
            if ($accountType === 'admin') {
                $deductPointsQuery = "UPDATE admins 
                                      SET point = point - :point 
                                      WHERE phone = :phone AND point >= :point";
            } elseif ($accountType === 'user') {
                $deductPointsQuery = "UPDATE users 
                                      SET point = point - :point 
                                      WHERE phone = :phone AND point >= :point";
            } else {
                $conn->rollBack();
                $utility->handleError(400, "نوع حساب نامعتبر است.");
                exit();
            }

            $deductPointsStmt = $conn->prepare($deductPointsQuery);
            $deductPointsStmt->bindValue(":point", $requestPoint, PDO::PARAM_INT);
            $deductPointsStmt->bindValue(":phone", $userPhone, PDO::PARAM_STR);

            if ($deductPointsStmt->execute() && $deductPointsStmt->rowCount() > 0) {
                $conn->commit();
                http_response_code(200);
                echo json_encode(["message" => "درخواست با موفقیت پرداخت شد."], JSON_UNESCAPED_UNICODE);
            } else {
                $conn->rollBack();
                $utility->handleError(500, "امکان کاهش امتیاز از حساب وجود ندارد.");
            }
        } else {
            $conn->rollBack();
            $utility->handleError(500, "خطا در پرداخت درخواست.");
        }
    } elseif ($data->action === 'reject') {
        // به‌روزرسانی وضعیت پرداخت به "رد شده"
        $updateQuery = "UPDATE transaction 
                        SET payment_status = 'رد شده', 
                            description = :description 
                        WHERE tracking_code = :tracking_code 
                          AND payment_status = 'در انتظار'";
        $updateStmt = $conn->prepare($updateQuery);
        $updateStmt->bindValue(":tracking_code", $data->tracking_code, PDO::PARAM_STR);
        $updateStmt->bindValue(":description", $data->description ?? '', PDO::PARAM_STR);

        if ($updateStmt->execute() && $updateStmt->rowCount() > 0) {
            $conn->commit();
            http_response_code(200);
            echo json_encode(["message" => "درخواست با موفقیت رد شد."], JSON_UNESCAPED_UNICODE);
        } else {
            $conn->rollBack();
            $utility->handleError(500, "خطا در رد کردن درخواست.");
        }
    } else {
        $utility->handleError(400, "عملیات معتبری انتخاب نشده است. اعمال معتبر: 'accept' یا 'reject'.");
        exit();
    }
} catch (ExpiredException $e) {
    $utility->handleError(401, "توکن منقضی شده است.");
} catch (\Firebase\JWT\SignatureInvalidException $e) {
    $utility->handleError(401, "توکن نامعتبر است.");
} catch (Exception $e) {
    if ($conn->inTransaction()) {
        $conn->rollBack();
    }
    $utility->handleError(500, "خطایی در سرور رخ داده است.");
}
?>
