<?php
header("Content-Type: application/json; charset=UTF-8");
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST");

// فایل‌های مورد نیاز
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/config/core.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/config/Database.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/auth/class.Utility.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/libs/php-jwt-master/src/JWT.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/libs/php-jwt-master/src/ExpiredException.php';

use \Firebase\JWT\JWT;
use \Firebase\JWT\ExpiredException;

$utility = new Utility();

// دریافت توکن از هدر Authorization
$encodedString = $utility->getToken();
if (!$encodedString) {
    $utility->handleError(401, "توکن ارائه نشده است.");
    exit();
}

// دریافت داده‌های ورودی از بدنه درخواست
$data = json_decode(file_get_contents("php://input"));

// اعتبارسنجی داده‌های ورودی
if (
    empty($data->point) ||
    empty($data->card_number) ||
    empty($data->card_name) ||
    empty($data->bank_name)
) {
    $utility->handleError(400, "لطفاً تمام فیلدها را پر کنید.");
    exit();
}

try {
    // بررسی اعتبار توکن
    $decodedToken = JWT::decode($encodedString, JWT_KEY, ['HS256']);
    $decodedPayload = (array) $decodedToken->data;

    // سطوح دسترسی مجاز
    $allowedAccessLevels = ['manager', '1', '2', '3'];

    // بررسی دسترسی
    if (!isset($decodedPayload['access_level']) || !in_array($decodedPayload['access_level'], $allowedAccessLevels)) {
        $utility->handleError(403, "شما دسترسی لازم برای این عملیات را ندارید.");
        exit();
    }

    // اتصال به دیتابیس
    $database = new Database();
    $conn = $database->getConnection();

    // شناسه کاربر از توکن
    $userId = $decodedPayload['id'] ?? null;

    if (!$userId) {
        $utility->handleError(400, "شناسه کاربری در توکن وجود ندارد.");
        exit();
    }

    // دریافت اطلاعات کاربر (بازگرداندن phone و سایر اطلاعات) از جدول admins
    $userQuery = "SELECT full_name, phone, referral_code, point AS available_point FROM admins WHERE id = :id";
    $userStmt = $conn->prepare($userQuery);
    $userStmt->bindValue(":id", $userId, PDO::PARAM_INT);
    $userStmt->execute();
    $userData = $userStmt->fetch(PDO::FETCH_ASSOC);

    if (!$userData) {
        $utility->handleError(404, "کاربری با این شناسه یافت نشد.");
        exit();
    }

    // اطلاعات کاربر
    $fullName = $userData['full_name'] ?? '';
    $phone = $userData['phone'] ?? '';
    $referralCode = $userData['referral_code'] ?? '';
    $availablePoint = intval($userData['available_point'] ?? 0); // امتیاز موجود کاربر

    // بررسی وجود درخواست "در انتظار" برای کاربر
    $checkPendingRequestQuery = "SELECT id FROM transaction WHERE phone = :phone AND payment_status = 'در انتظار' LIMIT 1";
    $checkStmt = $conn->prepare($checkPendingRequestQuery);
    $checkStmt->bindValue(":phone", $phone, PDO::PARAM_STR);
    $checkStmt->execute();

    if ($checkStmt->fetch(PDO::FETCH_ASSOC)) {
        $utility->handleError(400, "شما دارای درخواستی 'در انتظار' هستید. تا زمانی که این درخواست پرداخت یا رد نشود، نمی‌توانید درخواست جدیدی ثبت کنید.");
        exit();
    }

    // اعتبارسنجی امتیاز درخواستی
    $requestedPoint = intval($data->point);

    if ($requestedPoint < 50) {
        $utility->handleError(400, "حداقل مقدار درخواست امتیاز 50 است.");
        exit();
    }

    if ($requestedPoint > $availablePoint) {
        $utility->handleError(400, "مقدار درخواستی از امتیاز موجود شما بیشتر است.");
        exit();
    }

    // تابع تولید کد پیگیری شش رقمی
    function generateUniqueTrackingCode($conn) {
        do {
            $trackingCode = str_pad(random_int(0, 999999), 6, '0', STR_PAD_LEFT); // تولید عدد شش رقمی
            $checkCodeQuery = "SELECT id FROM transaction WHERE tracking_code = :tracking_code LIMIT 1";
            $checkCodeStmt = $conn->prepare($checkCodeQuery);
            $checkCodeStmt->bindValue(":tracking_code", $trackingCode, PDO::PARAM_STR);
            $checkCodeStmt->execute();
            $isDuplicate = $checkCodeStmt->fetch(PDO::FETCH_ASSOC);
        } while ($isDuplicate); // اگر کد تکراری بود، دوباره تولید کن

        return $trackingCode;
    }

    // شروع تراکنش
    $conn->beginTransaction();

    // محاسبه مبلغ قابل پرداخت (هر ده امتیاز = 5000 ریال)
    $balance = intval(($requestedPoint / 10) * 50000); // حذف اعداد عشاری با intval()

    // تولید کد پیگیری شش رقمی
    $trackingCode = generateUniqueTrackingCode($conn);

    // درج اطلاعات درخواست برداشت امتیاز با تنظیم account_type به "admin"
    $insertQuery = "INSERT INTO transaction (
                        point,
                        balance,
                        card_number,
                        card_name,
                        bank_name,
                        date_request,
                        full_name,
                        phone,
                        referral_code,
                        payment_status,
                        tracking_code,
                        description,
                        account_type
                    ) VALUES (
                        :point,
                        :balance,
                        :card_number,
                        :card_name,
                        :bank_name,
                        NOW(),
                        :full_name,
                        :phone,
                        :referral_code,
                        'در انتظار',
                        :tracking_code,
                        :description,
                        'admin'
                    )";

    $insertStmt = $conn->prepare($insertQuery);
    $insertStmt->bindValue(":point", $requestedPoint, PDO::PARAM_INT);
    $insertStmt->bindValue(":balance", $balance, PDO::PARAM_INT);
    $insertStmt->bindValue(":card_number", $data->card_number, PDO::PARAM_STR);
    $insertStmt->bindValue(":card_name", $data->card_name, PDO::PARAM_STR);
    $insertStmt->bindValue(":bank_name", $data->bank_name, PDO::PARAM_STR);
    $insertStmt->bindValue(":full_name", $fullName, PDO::PARAM_STR);
    $insertStmt->bindValue(":phone", $phone, PDO::PARAM_STR);
    $insertStmt->bindValue(":referral_code", $referralCode, PDO::PARAM_STR);
    $insertStmt->bindValue(":tracking_code", $trackingCode, PDO::PARAM_STR);
    $insertStmt->bindValue(":description", $data->description ?? '', PDO::PARAM_STR);

    if ($insertStmt->execute()) {
        // بررسی تعداد ردیف‌های درج شده
        if ($insertStmt->rowCount() > 0) {
            $conn->commit();
            http_response_code(200);
            echo json_encode([
                "message" => "درخواست برداشت امتیاز شما با موفقیت ثبت شد.",
                "tracking_code" => $trackingCode,
                "balance" => $balance // مبلغ قابل پرداخت به ریال
            ], JSON_UNESCAPED_UNICODE);
        } else {
            $conn->rollBack();
            $utility->handleError(500, "خطا در ثبت درخواست برداشت امتیاز.");
        }
    } else {
        $conn->rollBack();
        $utility->handleError(500, "خطا در پردازش درخواست برداشت امتیاز.");
    }
} catch (ExpiredException $e) {
    $utility->handleError(401, "توکن منقضی شده است.");
} catch (\Firebase\JWT\SignatureInvalidException $e) {
    $utility->handleError(401, "توکن نامعتبر است.");
} catch (Exception $e) {
    if ($conn->inTransaction()) {
        $conn->rollBack();
    }
    $utility->handleError(500, "خطایی در سرور رخ داده است.");
}
?>
