<?php
header("Content-Type: application/json; charset=UTF-8");
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST");

require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/config/core.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/config/Database.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/auth/class.Utility.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/libs/php-jwt-master/src/JWT.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/jobfinder/src/libs/php-jwt-master/src/ExpiredException.php';

use Firebase\JWT\JWT;
use Firebase\JWT\ExpiredException;

$utility = new Utility();
$encodedString = $utility->getToken();

if (!$encodedString) {
    $utility->handleError(401, "توکن ارائه نشده است.");
    exit();
}

try {
    $decodedToken = JWT::decode($encodedString, JWT_KEY, ['HS256']);
    $decodedPayload = (array) $decodedToken->data;

    if (empty($decodedPayload['user_phone'])) {
        $utility->handleError(400, "شماره تلفن در توکن وجود ندارد.");
        exit();
    }

    // دریافت اطلاعات ورودی به صورت JSON
    $data = json_decode(file_get_contents("php://input"), false, 512, JSON_THROW_ON_ERROR);

    if (
        empty($data->point) || empty($data->card_number) || empty($data->card_name) ||
        empty($data->bank_name) || empty($data->full_name) || empty($data->referral_code)
    ) {
        $utility->handleError(400, "لطفاً تمام فیلدها را پر کنید.");
        exit();
    }

    if (!ctype_digit($data->card_number)) {
        $utility->handleError(400, "شماره کارت فقط باید شامل اعداد باشد.");
        exit();
    }

    $requestedPoint = intval($data->point);
    if ($requestedPoint < 50) {
        $utility->handleError(400, "حداقل مقدار درخواست امتیاز 50 است.");
        exit();
    }

    $database = new Database();
    $conn = $database->getConnection();
    $userPhone = $decodedPayload['user_phone'];

    // شروع تراکنش
    $conn->beginTransaction();

    // بررسی وجود درخواست پرداخت در وضعیت 'در انتظار' برای کاربر
    $checkQuery = "
        SELECT u.point, EXISTS (SELECT 1 FROM transaction WHERE phone = :phone AND payment_status = 'در انتظار' LIMIT 1) AS has_pending_request
        FROM users u WHERE u.phone = :phone";
    $checkStmt = $conn->prepare($checkQuery);
    $checkStmt->bindValue(":phone", $userPhone, PDO::PARAM_STR);
    $checkStmt->execute();
    $result = $checkStmt->fetch(PDO::FETCH_ASSOC);

    if (!$result) {
        $utility->handleError(400, "کاربر یافت نشد.");
        exit();
    }

    if ($result['has_pending_request']) {
        $utility->handleError(400, "شما دارای درخواستی 'در انتظار' هستید.");
        exit();
    }

    if ($result['point'] < $requestedPoint) {
        $utility->handleError(400, "موجودی امتیاز شما کافی نیست.");
        exit();
    }

    // اعتبارسنجی رفرال کد
    $checkReferralStmt = $conn->prepare("SELECT EXISTS (SELECT 1 FROM users WHERE referral_code = :referral_code LIMIT 1) AS is_valid");
    $checkReferralStmt->bindValue(":referral_code", $data->referral_code, PDO::PARAM_STR);
    $checkReferralStmt->execute();
    $referralValid = $checkReferralStmt->fetch(PDO::FETCH_ASSOC)['is_valid'];

    if (!$referralValid) {
        $utility->handleError(400, "رفرال کد معتبر نیست.");
        exit();
    }

    // تابع تولید کد پیگیری یکتا (شش رقمی)
    function generateUniqueTrackingCode($conn) {
        do {
            $trackingCode = str_pad(random_int(0, 999999), 6, '0', STR_PAD_LEFT);
            $stmt = $conn->prepare("SELECT 1 FROM transaction WHERE tracking_code = :tracking_code LIMIT 1");
            $stmt->bindValue(":tracking_code", $trackingCode, PDO::PARAM_STR);
            $stmt->execute();
        } while ($stmt->fetch(PDO::FETCH_ASSOC));
        return $trackingCode;
    }

    // محاسبه مبلغ قابل پرداخت (هر ده امتیاز = 50000 ریال)
    $balance = intval(($requestedPoint / 10) * 50000);
    $formattedBalance = number_format($balance);
    $trackingCode = generateUniqueTrackingCode($conn);

    // درج درخواست برداشت امتیاز با تنظیم account_type به صورت ثابت 'user'
    $insertStmt = $conn->prepare("
        INSERT INTO transaction (
            point,
            balance,
            card_number,
            card_name,
            bank_name,
            date_request,
            full_name,
            phone,
            payment_status,
            tracking_code,
            description,
            referral_code,
            account_type
        ) VALUES (
            :point,
            :balance,
            :card_number,
            :card_name,
            :bank_name,
            NOW(),
            :full_name,
            :phone,
            'در انتظار',
            :tracking_code,
            :description,
            :referral_code,
            'user'
        )
    ");

    $insertStmt->execute([
        ":point" => $requestedPoint,
        ":balance" => $balance,
        ":card_number" => $data->card_number,
        ":card_name" => $data->card_name,
        ":bank_name" => $data->bank_name,
        ":full_name" => $data->full_name,
        ":phone" => $userPhone,
        ":tracking_code" => $trackingCode,
        ":description" => $data->description ?? '',
        ":referral_code" => $data->referral_code
    ]);

    if ($insertStmt->rowCount() > 0) {
        $conn->commit();
        http_response_code(200);
        echo json_encode([
            "message" => "درخواست برداشت امتیاز شما با موفقیت ثبت شد.",
            "tracking_code" => $trackingCode,
            "balance" => $formattedBalance
        ], JSON_UNESCAPED_UNICODE);
    } else {
        throw new Exception("خطا در ثبت درخواست.");
    }
} catch (ExpiredException $e) {
    $utility->handleError(401, "توکن منقضی شده است.");
} catch (\Firebase\JWT\SignatureInvalidException $e) {
    $utility->handleError(401, "توکن نامعتبر است.");
} catch (JsonException $e) {
    $utility->handleError(400, "فرمت JSON نامعتبر است.");
} catch (Exception $e) {
    $utility->handleError(500, "خطایی در سرور رخ داده است.");
}
?>
